# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagementApiOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_get(self, resource_group):
        response = self.client.api.get(
            resource_group_name=resource_group.name,
            service_name="str",
            api_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_get_entity_tag(self, resource_group):
        response = self.client.api.get_entity_tag(
            resource_group_name=resource_group.name,
            service_name="str",
            api_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_begin_create_or_update(self, resource_group):
        response = self.client.api.begin_create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            api_id="str",
            parameters={
                "properties": {
                    "path": "str",
                    "apiRevision": "str",
                    "apiRevisionDescription": "str",
                    "apiType": "str",
                    "apiVersion": "str",
                    "apiVersionDescription": "str",
                    "apiVersionSet": {
                        "description": "str",
                        "id": "str",
                        "name": "str",
                        "versionHeaderName": "str",
                        "versionQueryName": "str",
                        "versioningScheme": "str",
                    },
                    "apiVersionSetId": "str",
                    "authenticationSettings": {
                        "oAuth2": {"authorizationServerId": "str", "scope": "str"},
                        "oAuth2AuthenticationSettings": [{"authorizationServerId": "str", "scope": "str"}],
                        "openid": {"bearerTokenSendingMethods": ["str"], "openidProviderId": "str"},
                        "openidAuthenticationSettings": [
                            {"bearerTokenSendingMethods": ["str"], "openidProviderId": "str"}
                        ],
                    },
                    "contact": {"email": "str", "name": "str", "url": "str"},
                    "description": "str",
                    "displayName": "str",
                    "format": "str",
                    "isCurrent": bool,
                    "isOnline": bool,
                    "license": {"name": "str", "url": "str"},
                    "protocols": ["str"],
                    "provisioningState": "str",
                    "serviceUrl": "str",
                    "sourceApiId": "str",
                    "subscriptionKeyParameterNames": {"header": "str", "query": "str"},
                    "subscriptionRequired": bool,
                    "termsOfServiceUrl": "str",
                    "translateRequiredQueryParameters": "str",
                    "type": "str",
                    "value": "str",
                    "wsdlSelector": {"wsdlEndpointName": "str", "wsdlServiceName": "str"},
                }
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_update(self, resource_group):
        response = self.client.api.update(
            resource_group_name=resource_group.name,
            service_name="str",
            api_id="str",
            parameters={
                "properties": {
                    "apiRevision": "str",
                    "apiRevisionDescription": "str",
                    "apiVersion": "str",
                    "apiVersionDescription": "str",
                    "apiVersionSetId": "str",
                    "authenticationSettings": {
                        "oAuth2": {"authorizationServerId": "str", "scope": "str"},
                        "oAuth2AuthenticationSettings": [{"authorizationServerId": "str", "scope": "str"}],
                        "openid": {"bearerTokenSendingMethods": ["str"], "openidProviderId": "str"},
                        "openidAuthenticationSettings": [
                            {"bearerTokenSendingMethods": ["str"], "openidProviderId": "str"}
                        ],
                    },
                    "contact": {"email": "str", "name": "str", "url": "str"},
                    "description": "str",
                    "displayName": "str",
                    "isCurrent": bool,
                    "isOnline": bool,
                    "license": {"name": "str", "url": "str"},
                    "path": "str",
                    "protocols": ["str"],
                    "serviceUrl": "str",
                    "subscriptionKeyParameterNames": {"header": "str", "query": "str"},
                    "subscriptionRequired": bool,
                    "termsOfServiceUrl": "str",
                    "type": "str",
                }
            },
            etag="str",
            match_condition=~azure.core.MatchConditions,
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_begin_delete(self, resource_group):
        response = self.client.api.begin_delete(
            resource_group_name=resource_group.name,
            service_name="str",
            api_id="str",
            etag="str",
            match_condition=~azure.core.MatchConditions,
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_list_by_service(self, resource_group):
        response = self.client.api.list_by_service(
            resource_group_name=resource_group.name,
            service_name="str",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_api_list_by_tags(self, resource_group):
        response = self.client.api.list_by_tags(
            resource_group_name=resource_group.name,
            service_name="str",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...



import argparse
import os
import tempfile
import typing
import sys
import subprocess
import json
import pathlib

from typing import Optional, List
from subprocess import CalledProcessError

from .Check import Check
from ci_tools.functions import install_into_venv
from ci_tools.scenario.generation import create_package_and_install
from ci_tools.variables import discover_repo_root, in_ci, set_envvar_defaults
from ci_tools.environment_exclusions import is_check_enabled, is_typing_ignored
from ci_tools.functions import get_pip_command
from ci_tools.logging import logger

PYRIGHT_VERSION = "1.1.407"
REPO_ROOT = discover_repo_root()


class verifytypes(Check):
    def __init__(self) -> None:
        super().__init__()

    def register(
        self, subparsers: "argparse._SubParsersAction", parent_parsers: Optional[List[argparse.ArgumentParser]] = None
    ) -> None:
        """Register the verifytypes check. The verifytypes check installs verifytypes and runs verifytypes against the target package."""
        parents = parent_parsers or []
        p = subparsers.add_parser(
            "verifytypes", parents=parents, help="Run the verifytypes check to verify type completeness of a package."
        )
        p.set_defaults(func=self.run)

    def run(self, args: argparse.Namespace) -> int:
        """Run the verifytypes check command."""
        logger.info("Running verifytypes check...")

        set_envvar_defaults()
        targeted = self.get_targeted_directories(args)

        results: List[int] = []

        for parsed in targeted:
            if os.getcwd() != parsed.folder:
                os.chdir(parsed.folder)
            package_dir = parsed.folder
            package_name = parsed.name
            module = parsed.namespace
            executable, staging_directory = self.get_executable(
                args.isolate,
                args.command,
                sys.executable,
                package_dir,
                python_version=getattr(args, "python_version", None),
            )
            logger.info(f"Processing {package_name} for verifytypes check")

            self.install_dev_reqs(executable, args, package_dir)

            # install pyright
            try:
                install_into_venv(executable, [f"pyright=={PYRIGHT_VERSION}"], package_dir)
            except CalledProcessError as e:
                logger.error(f"Failed to install pyright: {e}")
                return e.returncode

            create_package_and_install(
                distribution_directory=staging_directory,
                target_setup=package_dir,
                skip_install=False,
                cache_dir=None,
                work_dir=staging_directory,
                force_create=False,
                package_type="sdist",
                pre_download_disabled=False,
                python_executable=executable,
            )

            if in_ci():
                if not is_check_enabled(package_dir, "verifytypes") or is_typing_ignored(package_name):
                    logger.info(
                        f"{package_name} opts-out of verifytypes check. See https://aka.ms/python/typing-guide for information."
                    )
                    continue

            commands = [
                executable,
                "-m",
                "pyright",
                "--verifytypes",
                module,
                "--ignoreexternal",
                "--outputjson",
            ]

            # get type completeness score from current code
            score_from_current = self.get_type_complete_score(executable, commands, package_dir, check_pytyped=True)
            if score_from_current == -1.0:
                results.append(1)
                continue

            # show output
            try:
                response = self.run_venv_command(executable, commands[1:-1], package_dir, check=True)
                logger.info(response.stdout)
            except subprocess.CalledProcessError as e:
                logger.warning(
                    f"verifytypes reported issues: {e.stdout}"
                )  # we don't fail on verifytypes, only if type completeness score worsens from main

            if in_ci():
                # get type completeness score from main
                logger.info("Getting the type completeness score from the code in main...")
                if self.install_from_main(os.path.abspath(package_dir), executable) > 0:
                    continue

                score_from_main = self.get_type_complete_score(executable, commands, package_dir)
                if score_from_main == -1.0:
                    results.append(1)
                    continue

                score_from_main_rounded = round(score_from_main * 100, 1)
                score_from_current_rounded = round(score_from_current * 100, 1)
                logger.info("\n-----Type completeness score comparison-----\n")
                logger.info(f"Score in main: {score_from_main_rounded}%")
                # Give a 5% buffer for type completeness score to decrease
                if score_from_current_rounded < score_from_main_rounded - 5:
                    logger.error(
                        f"\nERROR: The type completeness score of {package_name} has significantly decreased compared to the score in main. "
                        f"See the above output for areas to improve. See https://aka.ms/python/typing-guide for information."
                    )
                    results.append(1)
        return max(results) if results else 0

    def install_from_main(self, setup_path: str, python_executable: Optional[str] = None) -> int:
        path = pathlib.Path(setup_path)

        subdirectory = path.relative_to(REPO_ROOT)
        cwd = os.getcwd()
        with tempfile.TemporaryDirectory() as temp_dir_name:
            os.chdir(temp_dir_name)
            try:
                subprocess.check_call(["git", "init"], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
                subprocess.check_call(
                    [
                        "git",
                        "clone",
                        "--no-checkout",
                        "https://github.com/Azure/azure-sdk-for-python.git",
                        "--depth",
                        "1",
                    ],
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.STDOUT,
                )
                os.chdir("azure-sdk-for-python")
                subprocess.check_call(
                    ["git", "sparse-checkout", "init", "--cone"], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT
                )
                subprocess.check_call(
                    ["git", "sparse-checkout", "set", subdirectory], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT
                )
                subprocess.check_call(["git", "checkout", "main"], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)

                if not os.path.exists(os.path.join(os.getcwd(), subdirectory)):
                    # code is not checked into main yet, nothing to compare
                    logger.info(f"{subdirectory} is not checked into main, nothing to compare.")
                    return 1

                os.chdir(subdirectory)

                command = get_pip_command(python_executable) + ["install", ".", "--force-reinstall"]

                # When using uv, add --no-sources to ignore [tool.uv.sources] relative paths
                # that can't resolve in a sparse checkout, and --python to target the correct venv.
                if command[0] == "uv":
                    command += ["--no-sources"]
                    if python_executable:
                        command += ["--python", python_executable]

                subprocess.check_call(command, stdout=subprocess.DEVNULL)
            finally:
                os.chdir(cwd)  # allow temp dir to be deleted
            return 0

    def get_type_complete_score(
        self, executable, commands: typing.List[str], cwd: str, check_pytyped: bool = False
    ) -> float:
        try:
            response = self.run_venv_command(executable, commands[1:], cwd, check=True)
        except subprocess.CalledProcessError as e:
            if e.returncode != 1:
                logger.error(
                    f"Running verifytypes failed: {e.stderr}. See https://aka.ms/python/typing-guide for information."
                )
                return -1.0
            try:
                report = json.loads(e.output)
            except (json.JSONDecodeError, TypeError):
                logger.error(
                    f"pyright --verifytypes exited with code 1 but did not produce valid JSON output.\n"
                    f"stdout: {e.output}\n"
                    f"stderr: {e.stderr}\n"
                    f"Re-running without --outputjson for diagnostic output..."
                )
                non_json_commands = [c for c in commands[1:] if c != "--outputjson"] + ["--verbose"]
                diag = self.run_venv_command(executable, non_json_commands, cwd, check=False)
                logger.error(f"Diagnostic pyright stdout:\n{diag.stdout}")
                if diag.stderr:
                    logger.error(f"Diagnostic pyright stderr:\n{diag.stderr}")
                return -1.0
            if check_pytyped:
                pytyped_present = report["typeCompleteness"].get("pyTypedPath", None)
                if not pytyped_present:
                    logger.error(f"No py.typed file was found. See https://aka.ms/python/typing-guide for information.")
                    return -1.0
            return report["typeCompleteness"]["completenessScore"]

        # library scores 100%
        try:
            report = json.loads(response.stdout)
        except (json.JSONDecodeError, TypeError):
            logger.error(
                f"pyright --verifytypes exited successfully but did not produce valid JSON output.\n"
                f"stdout: {response.stdout}\n"
                f"stderr: {response.stderr}\n"
                f"Re-running without --outputjson for diagnostic output..."
            )
            non_json_commands = [c for c in commands[1:] if c != "--outputjson"] + ["--verbose"]
            diag = self.run_venv_command(executable, non_json_commands, cwd, check=False)
            logger.error(f"Diagnostic pyright stdout:\n{diag.stdout}")
            if diag.stderr:
                logger.error(f"Diagnostic pyright stderr:\n{diag.stderr}")
            return -1.0
        return report["typeCompleteness"]["completenessScore"]



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billingbenefits
# USAGE
    python savings_plan_order_alias_create_single_scope.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingBenefitsMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.savings_plan_order_alias.begin_create(
        savings_plan_order_alias_name="spAlias123",
        body={
            "properties": {
                "appliedScopeProperties": {"subscriptionId": "/subscriptions/30000000-0000-0000-0000-000000000000"},
                "appliedScopeType": "Single",
                "billingPlan": "P1M",
                "billingScopeId": "/providers/Microsoft.Billing/billingAccounts/1234567/billingSubscriptions/30000000-0000-0000-0000-000000000000",
                "commitment": {"amount": 0.001, "currencyCode": "USD", "grain": "Hourly"},
                "displayName": "Compute_SavingsPlan_10-28-2022_16-38",
                "term": "P3Y",
            },
            "sku": {"name": "Compute_Savings_Plan"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-12-01-preview/SavingsPlanOrderAliasCreateSingleScope.json
if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Unit tests for SSE encoding helpers."""

from __future__ import annotations

from azure.ai.agentserver.responses.streaming import _sse


class _FakeEvent:
    def __init__(self, type: str, sequence_number: int, text: str) -> None:
        self.type = type
        self.sequence_number = sequence_number
        self.text = text


def test_sse_writer__encodes_event_and_data_lines_with_separator() -> None:
    event = _FakeEvent(type="response.created", sequence_number=0, text="hello")

    encoded = _sse.encode_sse_event(event)  # type: ignore[arg-type]
    assert encoded.startswith("event: response.created\n")
    assert "data:" in encoded
    assert encoded.endswith("\n\n")


def test_sse_writer__encodes_multiline_text_as_single_data_line() -> None:
    event = _FakeEvent(type="response.output_text.delta", sequence_number=1, text="line1\nline2")

    encoded = _sse.encode_sse_event(event)  # type: ignore[arg-type]
    # Spec requires a single data: line with JSON payload — no extra data: lines
    assert encoded.count("data: ") == 1
    assert "data: line1" not in encoded
    assert r"line1\nline2" in encoded


def test_sse_writer__keep_alive_comment_frame_format() -> None:
    keep_alive_frame = _sse.encode_keep_alive_comment()  # type: ignore[attr-defined]
    assert keep_alive_frame == ": keep-alive\n\n"


def test_sse_writer__injects_monotonic_sequence_numbers() -> None:
    import json as _json

    _sse.new_stream_counter()

    first_event = _FakeEvent(type="response.created", sequence_number=-1, text="a")
    second_event = _FakeEvent(type="response.in_progress", sequence_number=-1, text="b")

    encoded_first = _sse.encode_sse_event(first_event)  # type: ignore[arg-type]
    encoded_second = _sse.encode_sse_event(second_event)  # type: ignore[arg-type]

    def _extract_sequence_number(encoded: str) -> int:
        data_line = next(line for line in encoded.splitlines() if line.startswith("data:"))
        payload = _json.loads(data_line[len("data:") :].strip())
        return int(payload["sequence_number"])

    seq_first = _extract_sequence_number(encoded_first)
    seq_second = _extract_sequence_number(encoded_second)

    assert seq_first == 0, f"first sequence_number must be 0 for a new stream, got {seq_first}"
    assert seq_second == 1, f"second sequence_number must be 1 for a new stream, got {seq_second}"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.web.aio import WebSiteManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestWebSiteManagementDiagnosticsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(WebSiteManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_detector_response(self, resource_group):
        response = await self.client.diagnostics.get_site_detector_response(
            resource_group_name=resource_group.name,
            site_name="str",
            detector_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_detector_responses(self, resource_group):
        response = self.client.diagnostics.list_site_detector_responses(
            resource_group_name=resource_group.name,
            site_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_hosting_environment_detector_response(self, resource_group):
        response = await self.client.diagnostics.get_hosting_environment_detector_response(
            resource_group_name=resource_group.name,
            name="str",
            detector_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_hosting_environment_detector_responses(self, resource_group):
        response = self.client.diagnostics.list_hosting_environment_detector_responses(
            resource_group_name=resource_group.name,
            name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_detector_response_slot(self, resource_group):
        response = await self.client.diagnostics.get_site_detector_response_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            detector_name="str",
            slot="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_detector_responses_slot(self, resource_group):
        response = self.client.diagnostics.list_site_detector_responses_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            slot="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_diagnostic_category(self, resource_group):
        response = await self.client.diagnostics.get_site_diagnostic_category(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_diagnostic_categories(self, resource_group):
        response = self.client.diagnostics.list_site_diagnostic_categories(
            resource_group_name=resource_group.name,
            site_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_diagnostic_category_slot(self, resource_group):
        response = await self.client.diagnostics.get_site_diagnostic_category_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            slot="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_diagnostic_categories_slot(self, resource_group):
        response = self.client.diagnostics.list_site_diagnostic_categories_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            slot="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_analysis(self, resource_group):
        response = await self.client.diagnostics.get_site_analysis(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            analysis_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_analyses(self, resource_group):
        response = self.client.diagnostics.list_site_analyses(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_execute_site_analysis(self, resource_group):
        response = await self.client.diagnostics.execute_site_analysis(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            analysis_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_analysis_slot(self, resource_group):
        response = await self.client.diagnostics.get_site_analysis_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            analysis_name="str",
            slot="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_analyses_slot(self, resource_group):
        response = self.client.diagnostics.list_site_analyses_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            slot="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_execute_site_analysis_slot(self, resource_group):
        response = await self.client.diagnostics.execute_site_analysis_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            analysis_name="str",
            slot="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_detector(self, resource_group):
        response = await self.client.diagnostics.get_site_detector(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            detector_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_detectors(self, resource_group):
        response = self.client.diagnostics.list_site_detectors(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_execute_site_detector(self, resource_group):
        response = await self.client.diagnostics.execute_site_detector(
            resource_group_name=resource_group.name,
            site_name="str",
            detector_name="str",
            diagnostic_category="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_get_site_detector_slot(self, resource_group):
        response = await self.client.diagnostics.get_site_detector_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            detector_name="str",
            slot="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_list_site_detectors_slot(self, resource_group):
        response = self.client.diagnostics.list_site_detectors_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            diagnostic_category="str",
            slot="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_diagnostics_execute_site_detector_slot(self, resource_group):
        response = await self.client.diagnostics.execute_site_detector_slot(
            resource_group_name=resource_group.name,
            site_name="str",
            detector_name="str",
            diagnostic_category="str",
            slot="str",
        )

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.textanalytics.authoring.aio import TextAuthoringClient
from azure.ai.textanalytics.authoring.models import CreateDeploymentDetails

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return TextAuthoringClient(endpoint, AzureKeyCredential(key))  # type: ignore[arg-type]

class TestConversationsDeleteDeploymentAsync(TestConversations):

    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_deploy_project_async(self, authoring_endpoint, authoring_key):
        async with TextAuthoringClient(authoring_endpoint, AzureKeyCredential(authoring_key)) as client:
            project_name = "single-class-project"
            deployment_name = "deployment0902"
            trained_model_label = "model5"

            project_client = client.get_project_client(project_name)

            # Build request body for deployment
            details = CreateDeploymentDetails(trained_model_label=trained_model_label)

            # Act: begin deploy and wait for completion
            poller = await project_client.deployment.begin_deploy_project(
                deployment_name=deployment_name,
                body=details,
            )
            try:
                await poller.result()
            except HttpResponseError as e:
                msg = getattr(getattr(e, "error", None), "message", str(e))
                print(f"Operation failed: {msg}")
                raise

            # If we get here, the deploy succeeded
            print(f"Deploy project completed. done={poller.done()} status={poller.status()}")

    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_delete_deployment_async(self, authoring_endpoint, authoring_key):
        async with TextAuthoringClient(authoring_endpoint, AzureKeyCredential(authoring_key)) as client:
            project_name = "single-class-project"
            deployment_name = "deployment0902"

            project_client = client.get_project_client(project_name)

            # Act: begin delete and wait for completion
            poller = await project_client.deployment.begin_delete_deployment(deployment_name=deployment_name)

            try:
                await poller.result()
            except HttpResponseError as e:
                msg = getattr(getattr(e, "error", None), "message", str(e))
                print(f"Operation failed: {msg}")
                raise

            # If we get here, the delete succeeded
            print(f"Delete completed. done={poller.done()} status={poller.status()}")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient
from azure.mgmt.core.policies import AsyncARMAutoResourceProviderRegistrationPolicy

from .. import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import BillingManagementClientConfiguration
from .operations import (
    AddressOperations,
    AgreementsOperations,
    AssociatedTenantsOperations,
    AvailableBalancesOperations,
    BillingAccountsOperations,
    BillingPermissionsOperations,
    BillingProfilesOperations,
    BillingPropertyOperations,
    BillingRequestsOperations,
    BillingRoleAssignmentsOperations,
    BillingRoleDefinitionOperations,
    BillingSubscriptionsAliasesOperations,
    BillingSubscriptionsOperations,
    CustomersOperations,
    DepartmentsOperations,
    EnrollmentAccountsOperations,
    InvoiceSectionsOperations,
    InvoicesOperations,
    Operations,
    PartnerTransfersOperations,
    PaymentMethodsOperations,
    PoliciesOperations,
    ProductsOperations,
    RecipientTransfersOperations,
    ReservationOrdersOperations,
    ReservationsOperations,
    SavingsPlanOrdersOperations,
    SavingsPlansOperations,
    TransactionsOperations,
    TransfersOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential


class BillingManagementClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Billing Client.

    :ivar agreements: AgreementsOperations operations
    :vartype agreements: azure.mgmt.billing.aio.operations.AgreementsOperations
    :ivar associated_tenants: AssociatedTenantsOperations operations
    :vartype associated_tenants: azure.mgmt.billing.aio.operations.AssociatedTenantsOperations
    :ivar available_balances: AvailableBalancesOperations operations
    :vartype available_balances: azure.mgmt.billing.aio.operations.AvailableBalancesOperations
    :ivar billing_accounts: BillingAccountsOperations operations
    :vartype billing_accounts: azure.mgmt.billing.aio.operations.BillingAccountsOperations
    :ivar address: AddressOperations operations
    :vartype address: azure.mgmt.billing.aio.operations.AddressOperations
    :ivar billing_permissions: BillingPermissionsOperations operations
    :vartype billing_permissions: azure.mgmt.billing.aio.operations.BillingPermissionsOperations
    :ivar billing_profiles: BillingProfilesOperations operations
    :vartype billing_profiles: azure.mgmt.billing.aio.operations.BillingProfilesOperations
    :ivar billing_property: BillingPropertyOperations operations
    :vartype billing_property: azure.mgmt.billing.aio.operations.BillingPropertyOperations
    :ivar billing_requests: BillingRequestsOperations operations
    :vartype billing_requests: azure.mgmt.billing.aio.operations.BillingRequestsOperations
    :ivar billing_role_assignments: BillingRoleAssignmentsOperations operations
    :vartype billing_role_assignments:
     azure.mgmt.billing.aio.operations.BillingRoleAssignmentsOperations
    :ivar billing_role_definition: BillingRoleDefinitionOperations operations
    :vartype billing_role_definition:
     azure.mgmt.billing.aio.operations.BillingRoleDefinitionOperations
    :ivar savings_plan_orders: SavingsPlanOrdersOperations operations
    :vartype savings_plan_orders: azure.mgmt.billing.aio.operations.SavingsPlanOrdersOperations
    :ivar savings_plans: SavingsPlansOperations operations
    :vartype savings_plans: azure.mgmt.billing.aio.operations.SavingsPlansOperations
    :ivar billing_subscriptions: BillingSubscriptionsOperations operations
    :vartype billing_subscriptions:
     azure.mgmt.billing.aio.operations.BillingSubscriptionsOperations
    :ivar billing_subscriptions_aliases: BillingSubscriptionsAliasesOperations operations
    :vartype billing_subscriptions_aliases:
     azure.mgmt.billing.aio.operations.BillingSubscriptionsAliasesOperations
    :ivar customers: CustomersOperations operations
    :vartype customers: azure.mgmt.billing.aio.operations.CustomersOperations
    :ivar departments: DepartmentsOperations operations
    :vartype departments: azure.mgmt.billing.aio.operations.DepartmentsOperations
    :ivar enrollment_accounts: EnrollmentAccountsOperations operations
    :vartype enrollment_accounts: azure.mgmt.billing.aio.operations.EnrollmentAccountsOperations
    :ivar invoices: InvoicesOperations operations
    :vartype invoices: azure.mgmt.billing.aio.operations.InvoicesOperations
    :ivar invoice_sections: InvoiceSectionsOperations operations
    :vartype invoice_sections: azure.mgmt.billing.aio.operations.InvoiceSectionsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.billing.aio.operations.Operations
    :ivar payment_methods: PaymentMethodsOperations operations
    :vartype payment_methods: azure.mgmt.billing.aio.operations.PaymentMethodsOperations
    :ivar policies: PoliciesOperations operations
    :vartype policies: azure.mgmt.billing.aio.operations.PoliciesOperations
    :ivar products: ProductsOperations operations
    :vartype products: azure.mgmt.billing.aio.operations.ProductsOperations
    :ivar reservations: ReservationsOperations operations
    :vartype reservations: azure.mgmt.billing.aio.operations.ReservationsOperations
    :ivar reservation_orders: ReservationOrdersOperations operations
    :vartype reservation_orders: azure.mgmt.billing.aio.operations.ReservationOrdersOperations
    :ivar transactions: TransactionsOperations operations
    :vartype transactions: azure.mgmt.billing.aio.operations.TransactionsOperations
    :ivar transfers: TransfersOperations operations
    :vartype transfers: azure.mgmt.billing.aio.operations.TransfersOperations
    :ivar partner_transfers: PartnerTransfersOperations operations
    :vartype partner_transfers: azure.mgmt.billing.aio.operations.PartnerTransfersOperations
    :ivar recipient_transfers: RecipientTransfersOperations operations
    :vartype recipient_transfers: azure.mgmt.billing.aio.operations.RecipientTransfersOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID that uniquely identifies a billing subscription. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2024-04-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = BillingManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                AsyncARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncARMPipelineClient = AsyncARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.agreements = AgreementsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.associated_tenants = AssociatedTenantsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.available_balances = AvailableBalancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_accounts = BillingAccountsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.address = AddressOperations(self._client, self._config, self._serialize, self._deserialize)
        self.billing_permissions = BillingPermissionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_profiles = BillingProfilesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_property = BillingPropertyOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_requests = BillingRequestsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_role_assignments = BillingRoleAssignmentsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_role_definition = BillingRoleDefinitionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.savings_plan_orders = SavingsPlanOrdersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.savings_plans = SavingsPlansOperations(self._client, self._config, self._serialize, self._deserialize)
        self.billing_subscriptions = BillingSubscriptionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.billing_subscriptions_aliases = BillingSubscriptionsAliasesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.customers = CustomersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.departments = DepartmentsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.enrollment_accounts = EnrollmentAccountsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.invoices = InvoicesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.invoice_sections = InvoiceSectionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.payment_methods = PaymentMethodsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.policies = PoliciesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.products = ProductsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.reservations = ReservationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.reservation_orders = ReservationOrdersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.transactions = TransactionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.transfers = TransfersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_transfers = PartnerTransfersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.recipient_transfers = RecipientTransfersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import warnings

from ._version import VERSION
from ._call_automation_client import CallAutomationClient
from ._call_connection_client import CallConnectionClient
from ._models import (
    CallConnectionProperties,
    FileSource,
    TextSource,
    SsmlSource,
    RecognitionChoice,
    CallParticipant,
    RecordingProperties,
    AddParticipantResult,
    RemoveParticipantResult,
    TransferCallResult,
    MediaStreamingOptions,
    TranscriptionOptions,
    ChannelAffinity,
    MuteParticipantResult,
    SendDtmfTonesResult,
    CancelAddParticipantOperationResult,
    AzureBlobContainerRecordingStorage,
    AzureCommunicationsRecordingStorage,
    MediaStreamingSubscription,
    TranscriptionSubscription,
)
from ._shared.models import (
    CommunicationIdentifier,
    PhoneNumberIdentifier,
    MicrosoftTeamsAppIdentifier,
    MicrosoftTeamsUserIdentifier,
    CommunicationUserIdentifier,
    CommunicationIdentifierKind,
    CommunicationCloudEnvironment,
    UnknownIdentifier,
    TeamsExtensionUserProperties,
    TeamsExtensionUserIdentifier,
)
from ._generated.models._enums import (
    CallRejectReason,
    RecordingContent,
    RecordingChannel,
    RecordingFormat,
    RecordingKind,
    RecordingStorageKind,
    RecognizeInputType,
    MediaStreamingAudioChannelType,
    MediaStreamingContentType,
    MediaStreamingSubscriptionState,
    TranscriptionResultType,
    TranscriptionSubscriptionState,
    StreamingTransportType,
    DtmfTone,
    CallConnectionState,
    RecordingState,
    VoiceKind,
    AudioFormat,
)

__all__ = [
    # clients
    "CallAutomationClient",
    "CallConnectionClient",
    # models for input
    "FileSource",
    "TextSource",
    "SsmlSource",
    "RecognitionChoice",
    "ChannelAffinity",
    "MediaStreamingOptions",
    "TranscriptionOptions",
    "AzureBlobContainerRecordingStorage",
    "AzureCommunicationsRecordingStorage",
    # models for output
    "CallConnectionProperties",
    "MediaStreamingSubscription",
    "TranscriptionSubscription",
    "CallParticipant",
    "RecordingProperties",
    "AddParticipantResult",
    "RemoveParticipantResult",
    "TransferCallResult",
    "MuteParticipantResult",
    "SendDtmfTonesResult",
    "CancelAddParticipantOperationResult",
    # common ACS communication identifier
    "CommunicationIdentifier",
    "PhoneNumberIdentifier",
    "MicrosoftTeamsAppIdentifier",
    "MicrosoftTeamsUserIdentifier",
    "CommunicationUserIdentifier",
    "CommunicationIdentifierKind",
    "CommunicationCloudEnvironment",
    "UnknownIdentifier",
    "TeamsExtensionUserProperties",
    "TeamsExtensionUserIdentifier",
    # enums
    "CallRejectReason",
    "RecordingContent",
    "RecordingChannel",
    "RecordingFormat",
    "RecordingKind",
    "RecordingStorageKind",
    "RecognizeInputType",
    "MediaStreamingAudioChannelType",
    "MediaStreamingContentType",
    "MediaStreamingSubscriptionState",
    "TranscriptionResultType",
    "TranscriptionSubscriptionState",
    "StreamingTransportType",
    "DtmfTone",
    "CallConnectionState",
    "RecordingState",
    "VoiceKind",
    "AudioFormat",
]
__version__ = VERSION


def __getattr__(name):
    if name == "CallInvite":
        warnings.warn(
            "CallInvite is deprecated and should not be used. Please pass in keyword arguments directly.",
            DeprecationWarning,
        )
        from ._models import CallInvite

        return CallInvite
    if name == "GroupCallLocator":
        warnings.warn(
            "GroupCallLocator is deprecated and should not be used. Please pass in 'group_call_id' directly.",
            DeprecationWarning,
        )
        from ._models import GroupCallLocator

        return GroupCallLocator
    if name == "ServerCallLocator":
        warnings.warn(
            "ServerCallLocator is deprecated and should not be used. Please pass in 'server_call_id' directly.",
            DeprecationWarning,
        )
        from ._models import ServerCallLocator

        return ServerCallLocator
    if name == "RoomCallLocator":
        warnings.warn(
            "RoomCallLocator is deprecated and should not be used. Please pass in 'room_id' directly.",
            DeprecationWarning,
        )
        from ._models import RoomCallLocator

        return RoomCallLocator
    if name == "MicrosoftBotIdentifier":
        warnings.warn(
            f"{name} is deprecated and should not be used. Please use 'MicrosoftTeamsAppIdentifier' instead.",
            DeprecationWarning,
        )
        from ._shared.models import _MicrosoftBotIdentifier

        return _MicrosoftBotIdentifier
    raise AttributeError(f"module 'azure.communication.callautomation' has no attribute {name}")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    ActionStatus,
    BranchStatus,
    Capability,
    CapabilityListResult,
    CapabilityType,
    CapabilityTypeListResult,
    CapabilityTypePropertiesRuntimeProperties,
    ChaosExperimentAction,
    ChaosExperimentBranch,
    ChaosExperimentStep,
    ChaosTargetFilter,
    ChaosTargetListSelector,
    ChaosTargetQuerySelector,
    ChaosTargetSelector,
    ChaosTargetSimpleFilter,
    ChaosTargetSimpleFilterParameters,
    ContinuousAction,
    DelayAction,
    DiscreteAction,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    Experiment,
    ExperimentExecution,
    ExperimentExecutionActionTargetDetailsError,
    ExperimentExecutionActionTargetDetailsProperties,
    ExperimentExecutionDetails,
    ExperimentExecutionDetailsProperties,
    ExperimentExecutionDetailsPropertiesRunInformation,
    ExperimentExecutionListResult,
    ExperimentListResult,
    ExperimentUpdate,
    KeyValuePair,
    ManagedServiceIdentity,
    Operation,
    OperationDisplay,
    OperationListResult,
    OperationStatusResult,
    ProxyResource,
    Resource,
    StepStatus,
    SystemData,
    Target,
    TargetListResult,
    TargetReference,
    TargetType,
    TargetTypeListResult,
    TrackedResource,
    UserAssignedIdentity,
)

from ._chaos_management_client_enums import (  # type: ignore
    ActionType,
    CreatedByType,
    ExperimentActionType,
    FilterType,
    ManagedServiceIdentityType,
    Origin,
    ProvisioningState,
    SelectorType,
    TargetReferenceType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ActionStatus",
    "BranchStatus",
    "Capability",
    "CapabilityListResult",
    "CapabilityType",
    "CapabilityTypeListResult",
    "CapabilityTypePropertiesRuntimeProperties",
    "ChaosExperimentAction",
    "ChaosExperimentBranch",
    "ChaosExperimentStep",
    "ChaosTargetFilter",
    "ChaosTargetListSelector",
    "ChaosTargetQuerySelector",
    "ChaosTargetSelector",
    "ChaosTargetSimpleFilter",
    "ChaosTargetSimpleFilterParameters",
    "ContinuousAction",
    "DelayAction",
    "DiscreteAction",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "Experiment",
    "ExperimentExecution",
    "ExperimentExecutionActionTargetDetailsError",
    "ExperimentExecutionActionTargetDetailsProperties",
    "ExperimentExecutionDetails",
    "ExperimentExecutionDetailsProperties",
    "ExperimentExecutionDetailsPropertiesRunInformation",
    "ExperimentExecutionListResult",
    "ExperimentListResult",
    "ExperimentUpdate",
    "KeyValuePair",
    "ManagedServiceIdentity",
    "Operation",
    "OperationDisplay",
    "OperationListResult",
    "OperationStatusResult",
    "ProxyResource",
    "Resource",
    "StepStatus",
    "SystemData",
    "Target",
    "TargetListResult",
    "TargetReference",
    "TargetType",
    "TargetTypeListResult",
    "TrackedResource",
    "UserAssignedIdentity",
    "ActionType",
    "CreatedByType",
    "ExperimentActionType",
    "FilterType",
    "ManagedServiceIdentityType",
    "Origin",
    "ProvisioningState",
    "SelectorType",
    "TargetReferenceType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# pylint: disable=line-too-long,useless-suppression
from helpers import AuthoringTestHelper
from testcase import QuestionAnsweringAuthoringTestCase

from azure.core.credentials import AzureKeyCredential
from azure.ai.language.questionanswering.authoring import QuestionAnsweringAuthoringClient



class TestCreateAndDeploy(QuestionAnsweringAuthoringTestCase):
    def test_polling_interval(self, qna_authoring_creds):
        client = QuestionAnsweringAuthoringClient(
            qna_authoring_creds["endpoint"], AzureKeyCredential(qna_authoring_creds["key"])
        )
        # Default polling interval may change across previews; assert it is a positive int (previously 30) instead of a fixed legacy value
        assert isinstance(client._config.polling_interval, int) and client._config.polling_interval > 0
        client = QuestionAnsweringAuthoringClient(
            qna_authoring_creds["endpoint"], AzureKeyCredential(qna_authoring_creds["key"]), polling_interval=1
        )
        assert client._config.polling_interval == 1

    def test_create_project(self, recorded_test, qna_authoring_creds):  # type: ignore[name-defined] # pylint: disable=unused-argument
        client = QuestionAnsweringAuthoringClient(
            qna_authoring_creds["endpoint"], AzureKeyCredential(qna_authoring_creds["key"])
        )
        project_name = "IsaacNewton"
        client.create_project( # pylint: disable=no-value-for-parameter
            project_name=project_name,
            options={
                "description": "Biography of Sir Isaac Newton",
                "language": "en",
                "multilingualResource": True,
                "settings": {"defaultAnswer": "no answer"},
            },
        )
        found = any(p.get("projectName") == project_name for p in client.list_projects())
        assert found

    def test_deploy_project(self, recorded_test, qna_authoring_creds):  # type: ignore[name-defined] # pylint: disable=unused-argument
        client = QuestionAnsweringAuthoringClient(
            qna_authoring_creds["endpoint"], AzureKeyCredential(qna_authoring_creds["key"])
        )
        project_name = "IsaacNewton"
        AuthoringTestHelper.create_test_project(
            client,
            project_name=project_name,
            is_deployable=True,
            polling_interval=0 if self.is_playback else None, # pylint: disable=using-constant-test
        )
        deployment_poller = client.begin_deploy_project(
            project_name=project_name,
            deployment_name="production",
            polling_interval=0 if self.is_playback else None,  # pylint: disable=using-constant-test
        )
        # Preview LRO returns None; just ensure it completes without error
        deployment_poller.result()
        assert any(d.get("deploymentName") == "production" for d in client.list_deployments(project_name=project_name))



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------

from datetime import timedelta
from typing import Optional


def convert_timedelta_to_mins(
    duration: Optional[timedelta],
) -> Optional[int]:
    """
    Returns the total number of minutes contained in the duration.
    :param duration: Time duration
    :type duration: ~datetime.timedelta
    :rtype: int
    """
    return None if duration is None else int(duration.total_seconds() / 60)



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from collections import namedtuple
import functools
import os
import datetime
import logging

try:
    from azure.mgmt.resource.resources import ResourceManagementClient
except ImportError:
    pass

from . import AzureMgmtPreparer
from .exceptions import AzureTestError, ReservedResourceNameError
from .sanitizers import add_general_string_sanitizer


logging.getLogger().setLevel(logging.INFO)


RESOURCE_GROUP_PARAM = "resource_group"


FakeResource = namedtuple("FakeResource", ["name", "id"])


class ResourceGroupPreparer(AzureMgmtPreparer):
    def __init__(
        self,
        name_prefix="",
        use_cache=False,
        random_name_length=75,
        parameter_name=RESOURCE_GROUP_PARAM,
        parameter_name_for_location="location",
        location="westus",
        disable_recording=True,
        playback_fake_resource=None,
        client_kwargs=None,
        random_name_enabled=False,
        delete_after_tag_timedelta=datetime.timedelta(hours=8),
    ):
        super(ResourceGroupPreparer, self).__init__(
            name_prefix,
            random_name_length,
            disable_recording=disable_recording,
            playback_fake_resource=playback_fake_resource,
            client_kwargs=client_kwargs,
            random_name_enabled=random_name_enabled,
        )
        self.location = location
        self.parameter_name = parameter_name
        self.parameter_name_for_location = parameter_name_for_location
        env_value = os.environ.get("AZURE_RESOURCEGROUP_NAME", None)
        self._need_creation = True
        if env_value:
            self.resource_random_name = env_value
            self._need_creation = False
        if self.random_name_enabled:
            self.resource_moniker = self.name_prefix + "rgname"
        self.set_cache(use_cache, parameter_name, name_prefix)
        self.delete_after_tag_timedelta = delete_after_tag_timedelta

    def _prefix_name(self, name):
        name = "rgpy-" + name
        if len(name) > 90:
            name = name[:90]
        return name

    def create_resource(self, name, **kwargs):
        if self.is_live and self._need_creation:
            self.client = self.create_mgmt_client(ResourceManagementClient)
            parameters = {"location": self.location}
            expiry = datetime.datetime.utcnow() + self.delete_after_tag_timedelta
            parameters["tags"] = {"DeleteAfter": expiry.replace(microsecond=0).isoformat()}

            parameters["tags"]["BuildId"] = os.environ.get("BUILD_BUILDID", "local")
            parameters["tags"]["BuildJob"] = os.environ.get("AGENT_JOBNAME", "local")
            parameters["tags"]["BuildNumber"] = os.environ.get("BUILD_BUILDNUMBER", "local")
            parameters["tags"]["BuildReason"] = os.environ.get("BUILD_REASON", "local")
            try:
                # Prefixing all RGs created here with 'rgpy-' for tracing purposes
                name = self._prefix_name(name)
                logging.info(
                    "Attempting to create a Resource Group with name {} and parameters {}".format(name, parameters)
                )
                self.resource = self.client.resource_groups.create_or_update(name, parameters)
            except Exception as ex:
                if "ReservedResourceName" in str(ex):
                    raise ReservedResourceNameError(name)
                raise
        else:
            self.resource = self.resource or FakeResource(
                name=name,
                id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/" + name,
            )
        if name != self.moniker:
            add_general_string_sanitizer(target=name, value=self.moniker)
        return {
            self.parameter_name: self.resource,
            self.parameter_name_for_location: self.location,
        }

    def remove_resource(self, name, **kwargs):
        if self.is_live and self._need_creation:
            name = self._prefix_name(name)
            try:
                if "wait_timeout" in kwargs:
                    azure_poller = self.client.resource_groups.begin_delete(name)
                    azure_poller.wait(kwargs.get("wait_timeout"))
                    if azure_poller.done():
                        return
                    raise AzureTestError("Timed out waiting for resource group to be deleted.")
                else:
                    self.client.resource_groups.begin_delete(name, polling=False).result()
            except (
                Exception
            ) as err:  # NOTE: some track 1 libraries do not have azure-core installed. Cannot use HttpResponseError here
                logging.info("Failed to delete resource group with name {}".format(name))
                logging.info("{}".format(err))
                pass


RandomNameResourceGroupPreparer = functools.partial(ResourceGroupPreparer, random_name_enabled=True)
CachedResourceGroupPreparer = functools.partial(ResourceGroupPreparer, use_cache=True, random_name_enabled=True)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.batch.aio import BatchManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBatchManagementPoolOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(BatchManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_get(self, resource_group):
        response = await self.client.pool.get(
            resource_group_name=resource_group.name,
            account_name="str",
            pool_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_create(self, resource_group):
        response = await self.client.pool.create(
            resource_group_name=resource_group.name,
            account_name="str",
            pool_name="str",
            parameters={
                "etag": "str",
                "id": "str",
                "identity": {
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "name": "str",
                "properties": {
                    "allocationState": "str",
                    "allocationStateTransitionTime": "2020-02-20 00:00:00",
                    "applicationPackages": [{"id": "str", "version": "str"}],
                    "autoScaleRun": {
                        "evaluationTime": "2020-02-20 00:00:00",
                        "error": {"code": "str", "message": "str", "details": [...]},
                        "results": "str",
                    },
                    "creationTime": "2020-02-20 00:00:00",
                    "currentDedicatedNodes": 0,
                    "currentLowPriorityNodes": 0,
                    "deploymentConfiguration": {
                        "virtualMachineConfiguration": {
                            "imageReference": {
                                "communityGalleryImageId": "str",
                                "id": "str",
                                "offer": "str",
                                "publisher": "str",
                                "sharedGalleryImageId": "str",
                                "sku": "str",
                                "version": "str",
                            },
                            "nodeAgentSkuId": "str",
                            "containerConfiguration": {
                                "type": "str",
                                "containerImageNames": ["str"],
                                "containerRegistries": [
                                    {
                                        "identityReference": {"resourceId": "str"},
                                        "password": "str",
                                        "registryServer": "str",
                                        "username": "str",
                                    }
                                ],
                            },
                            "dataDisks": [
                                {
                                    "diskSizeGB": 0,
                                    "lun": 0,
                                    "caching": "str",
                                    "managedDisk": {
                                        "diskEncryptionSet": {"id": "str"},
                                        "securityProfile": {
                                            "diskEncryptionSet": {"id": "str"},
                                            "securityEncryptionType": "str",
                                        },
                                        "storageAccountType": "str",
                                    },
                                }
                            ],
                            "diskEncryptionConfiguration": {
                                "customerManagedKey": {
                                    "identityReference": {"resourceId": "str"},
                                    "keyUrl": "str",
                                    "rotationToLatestKeyVersionEnabled": bool,
                                },
                                "targets": ["str"],
                            },
                            "extensions": [
                                {
                                    "name": "str",
                                    "publisher": "str",
                                    "type": "str",
                                    "autoUpgradeMinorVersion": bool,
                                    "enableAutomaticUpgrade": bool,
                                    "protectedSettings": {},
                                    "provisionAfterExtensions": ["str"],
                                    "settings": {},
                                    "typeHandlerVersion": "str",
                                }
                            ],
                            "licenseType": "str",
                            "nodePlacementConfiguration": {"policy": "str"},
                            "osDisk": {
                                "caching": "str",
                                "diskSizeGB": 0,
                                "ephemeralOSDiskSettings": {"placement": "str"},
                                "managedDisk": {
                                    "diskEncryptionSet": {"id": "str"},
                                    "securityProfile": {
                                        "diskEncryptionSet": {"id": "str"},
                                        "securityEncryptionType": "str",
                                    },
                                    "storageAccountType": "str",
                                },
                                "writeAcceleratorEnabled": bool,
                            },
                            "securityProfile": {
                                "encryptionAtHost": bool,
                                "proxyAgentSettings": {
                                    "enabled": bool,
                                    "imds": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                                    "wireServer": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                                },
                                "securityType": "str",
                                "uefiSettings": {"secureBootEnabled": bool, "vTpmEnabled": bool},
                            },
                            "serviceArtifactReference": {"id": "str"},
                            "windowsConfiguration": {"enableAutomaticUpdates": bool},
                        }
                    },
                    "displayName": "str",
                    "interNodeCommunication": "str",
                    "lastModified": "2020-02-20 00:00:00",
                    "metadata": [{"name": "str", "value": "str"}],
                    "mountConfiguration": [
                        {
                            "azureBlobFileSystemConfiguration": {
                                "accountName": "str",
                                "containerName": "str",
                                "relativeMountPath": "str",
                                "accountKey": "str",
                                "blobfuseOptions": "str",
                                "identityReference": {"resourceId": "str"},
                                "sasKey": "str",
                            },
                            "azureFileShareConfiguration": {
                                "accountKey": "str",
                                "accountName": "str",
                                "azureFileUrl": "str",
                                "relativeMountPath": "str",
                                "mountOptions": "str",
                            },
                            "cifsMountConfiguration": {
                                "password": "str",
                                "relativeMountPath": "str",
                                "source": "str",
                                "userName": "str",
                                "mountOptions": "str",
                            },
                            "nfsMountConfiguration": {
                                "relativeMountPath": "str",
                                "source": "str",
                                "mountOptions": "str",
                            },
                        }
                    ],
                    "networkConfiguration": {
                        "dynamicVnetAssignmentScope": "str",
                        "enableAcceleratedNetworking": bool,
                        "endpointConfiguration": {
                            "inboundNatPools": [
                                {
                                    "backendPort": 0,
                                    "frontendPortRangeEnd": 0,
                                    "frontendPortRangeStart": 0,
                                    "name": "str",
                                    "protocol": "str",
                                    "networkSecurityGroupRules": [
                                        {
                                            "access": "str",
                                            "priority": 0,
                                            "sourceAddressPrefix": "str",
                                            "sourcePortRanges": ["str"],
                                        }
                                    ],
                                }
                            ]
                        },
                        "publicIPAddressConfiguration": {
                            "ipAddressIds": ["str"],
                            "ipFamilies": ["str"],
                            "ipTags": [{"ipTagType": "str", "tag": "str"}],
                            "provision": "str",
                        },
                        "subnetId": "str",
                    },
                    "provisioningState": "str",
                    "provisioningStateTransitionTime": "2020-02-20 00:00:00",
                    "resizeOperationStatus": {
                        "errors": [{"code": "str", "message": "str", "details": [...]}],
                        "nodeDeallocationOption": "str",
                        "resizeTimeout": "1 day, 0:00:00",
                        "startTime": "2020-02-20 00:00:00",
                        "targetDedicatedNodes": 0,
                        "targetLowPriorityNodes": 0,
                    },
                    "scaleSettings": {
                        "autoScale": {"formula": "str", "evaluationInterval": "1 day, 0:00:00"},
                        "fixedScale": {
                            "nodeDeallocationOption": "str",
                            "resizeTimeout": "1 day, 0:00:00",
                            "targetDedicatedNodes": 0,
                            "targetLowPriorityNodes": 0,
                        },
                    },
                    "startTask": {
                        "commandLine": "str",
                        "containerSettings": {
                            "imageName": "str",
                            "containerHostBatchBindMounts": [{"isReadOnly": bool, "source": "str"}],
                            "containerRunOptions": "str",
                            "registry": {
                                "identityReference": {"resourceId": "str"},
                                "password": "str",
                                "registryServer": "str",
                                "username": "str",
                            },
                            "workingDirectory": "str",
                        },
                        "environmentSettings": [{"name": "str", "value": "str"}],
                        "maxTaskRetryCount": 0,
                        "resourceFiles": [
                            {
                                "autoStorageContainerName": "str",
                                "blobPrefix": "str",
                                "fileMode": "str",
                                "filePath": "str",
                                "httpUrl": "str",
                                "identityReference": {"resourceId": "str"},
                                "storageContainerUrl": "str",
                            }
                        ],
                        "userIdentity": {"autoUser": {"elevationLevel": "str", "scope": "str"}, "userName": "str"},
                        "waitForSuccess": bool,
                    },
                    "taskSchedulingPolicy": {"nodeFillType": "str", "jobDefaultOrder": "str"},
                    "taskSlotsPerNode": 0,
                    "upgradePolicy": {
                        "mode": "str",
                        "automaticOSUpgradePolicy": {
                            "disableAutomaticRollback": bool,
                            "enableAutomaticOSUpgrade": bool,
                            "osRollingUpgradeDeferral": bool,
                            "useRollingUpgradePolicy": bool,
                        },
                        "rollingUpgradePolicy": {
                            "enableCrossZoneUpgrade": bool,
                            "maxBatchInstancePercent": 0,
                            "maxUnhealthyInstancePercent": 0,
                            "maxUnhealthyUpgradedInstancePercent": 0,
                            "pauseTimeBetweenBatches": "str",
                            "prioritizeUnhealthyInstances": bool,
                            "rollbackFailedInstancesOnPolicyBreach": bool,
                        },
                    },
                    "userAccounts": [
                        {
                            "name": "str",
                            "password": "str",
                            "elevationLevel": "str",
                            "linuxUserConfiguration": {"gid": 0, "sshPrivateKey": "str", "uid": 0},
                            "windowsUserConfiguration": {"loginMode": "str"},
                        }
                    ],
                    "vmSize": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_update(self, resource_group):
        response = await self.client.pool.update(
            resource_group_name=resource_group.name,
            account_name="str",
            pool_name="str",
            parameters={
                "etag": "str",
                "id": "str",
                "identity": {
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "name": "str",
                "properties": {
                    "allocationState": "str",
                    "allocationStateTransitionTime": "2020-02-20 00:00:00",
                    "applicationPackages": [{"id": "str", "version": "str"}],
                    "autoScaleRun": {
                        "evaluationTime": "2020-02-20 00:00:00",
                        "error": {"code": "str", "message": "str", "details": [...]},
                        "results": "str",
                    },
                    "creationTime": "2020-02-20 00:00:00",
                    "currentDedicatedNodes": 0,
                    "currentLowPriorityNodes": 0,
                    "deploymentConfiguration": {
                        "virtualMachineConfiguration": {
                            "imageReference": {
                                "communityGalleryImageId": "str",
                                "id": "str",
                                "offer": "str",
                                "publisher": "str",
                                "sharedGalleryImageId": "str",
                                "sku": "str",
                                "version": "str",
                            },
                            "nodeAgentSkuId": "str",
                            "containerConfiguration": {
                                "type": "str",
                                "containerImageNames": ["str"],
                                "containerRegistries": [
                                    {
                                        "identityReference": {"resourceId": "str"},
                                        "password": "str",
                                        "registryServer": "str",
                                        "username": "str",
                                    }
                                ],
                            },
                            "dataDisks": [
                                {
                                    "diskSizeGB": 0,
                                    "lun": 0,
                                    "caching": "str",
                                    "managedDisk": {
                                        "diskEncryptionSet": {"id": "str"},
                                        "securityProfile": {
                                            "diskEncryptionSet": {"id": "str"},
                                            "securityEncryptionType": "str",
                                        },
                                        "storageAccountType": "str",
                                    },
                                }
                            ],
                            "diskEncryptionConfiguration": {
                                "customerManagedKey": {
                                    "identityReference": {"resourceId": "str"},
                                    "keyUrl": "str",
                                    "rotationToLatestKeyVersionEnabled": bool,
                                },
                                "targets": ["str"],
                            },
                            "extensions": [
                                {
                                    "name": "str",
                                    "publisher": "str",
                                    "type": "str",
                                    "autoUpgradeMinorVersion": bool,
                                    "enableAutomaticUpgrade": bool,
                                    "protectedSettings": {},
                                    "provisionAfterExtensions": ["str"],
                                    "settings": {},
                                    "typeHandlerVersion": "str",
                                }
                            ],
                            "licenseType": "str",
                            "nodePlacementConfiguration": {"policy": "str"},
                            "osDisk": {
                                "caching": "str",
                                "diskSizeGB": 0,
                                "ephemeralOSDiskSettings": {"placement": "str"},
                                "managedDisk": {
                                    "diskEncryptionSet": {"id": "str"},
                                    "securityProfile": {
                                        "diskEncryptionSet": {"id": "str"},
                                        "securityEncryptionType": "str",
                                    },
                                    "storageAccountType": "str",
                                },
                                "writeAcceleratorEnabled": bool,
                            },
                            "securityProfile": {
                                "encryptionAtHost": bool,
                                "proxyAgentSettings": {
                                    "enabled": bool,
                                    "imds": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                                    "wireServer": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                                },
                                "securityType": "str",
                                "uefiSettings": {"secureBootEnabled": bool, "vTpmEnabled": bool},
                            },
                            "serviceArtifactReference": {"id": "str"},
                            "windowsConfiguration": {"enableAutomaticUpdates": bool},
                        }
                    },
                    "displayName": "str",
                    "interNodeCommunication": "str",
                    "lastModified": "2020-02-20 00:00:00",
                    "metadata": [{"name": "str", "value": "str"}],
                    "mountConfiguration": [
                        {
                            "azureBlobFileSystemConfiguration": {
                                "accountName": "str",
                                "containerName": "str",
                                "relativeMountPath": "str",
                                "accountKey": "str",
                                "blobfuseOptions": "str",
                                "identityReference": {"resourceId": "str"},
                                "sasKey": "str",
                            },
                            "azureFileShareConfiguration": {
                                "accountKey": "str",
                                "accountName": "str",
                                "azureFileUrl": "str",
                                "relativeMountPath": "str",
                                "mountOptions": "str",
                            },
                            "cifsMountConfiguration": {
                                "password": "str",
                                "relativeMountPath": "str",
                                "source": "str",
                                "userName": "str",
                                "mountOptions": "str",
                            },
                            "nfsMountConfiguration": {
                                "relativeMountPath": "str",
                                "source": "str",
                                "mountOptions": "str",
                            },
                        }
                    ],
                    "networkConfiguration": {
                        "dynamicVnetAssignmentScope": "str",
                        "enableAcceleratedNetworking": bool,
                        "endpointConfiguration": {
                            "inboundNatPools": [
                                {
                                    "backendPort": 0,
                                    "frontendPortRangeEnd": 0,
                                    "frontendPortRangeStart": 0,
                                    "name": "str",
                                    "protocol": "str",
                                    "networkSecurityGroupRules": [
                                        {
                                            "access": "str",
                                            "priority": 0,
                                            "sourceAddressPrefix": "str",
                                            "sourcePortRanges": ["str"],
                                        }
                                    ],
                                }
                            ]
                        },
                        "publicIPAddressConfiguration": {
                            "ipAddressIds": ["str"],
                            "ipFamilies": ["str"],
                            "ipTags": [{"ipTagType": "str", "tag": "str"}],
                            "provision": "str",
                        },
                        "subnetId": "str",
                    },
                    "provisioningState": "str",
                    "provisioningStateTransitionTime": "2020-02-20 00:00:00",
                    "resizeOperationStatus": {
                        "errors": [{"code": "str", "message": "str", "details": [...]}],
                        "nodeDeallocationOption": "str",
                        "resizeTimeout": "1 day, 0:00:00",
                        "startTime": "2020-02-20 00:00:00",
                        "targetDedicatedNodes": 0,
                        "targetLowPriorityNodes": 0,
                    },
                    "scaleSettings": {
                        "autoScale": {"formula": "str", "evaluationInterval": "1 day, 0:00:00"},
                        "fixedScale": {
                            "nodeDeallocationOption": "str",
                            "resizeTimeout": "1 day, 0:00:00",
                            "targetDedicatedNodes": 0,
                            "targetLowPriorityNodes": 0,
                        },
                    },
                    "startTask": {
                        "commandLine": "str",
                        "containerSettings": {
                            "imageName": "str",
                            "containerHostBatchBindMounts": [{"isReadOnly": bool, "source": "str"}],
                            "containerRunOptions": "str",
                            "registry": {
                                "identityReference": {"resourceId": "str"},
                                "password": "str",
                                "registryServer": "str",
                                "username": "str",
                            },
                            "workingDirectory": "str",
                        },
                        "environmentSettings": [{"name": "str", "value": "str"}],
                        "maxTaskRetryCount": 0,
                        "resourceFiles": [
                            {
                                "autoStorageContainerName": "str",
                                "blobPrefix": "str",
                                "fileMode": "str",
                                "filePath": "str",
                                "httpUrl": "str",
                                "identityReference": {"resourceId": "str"},
                                "storageContainerUrl": "str",
                            }
                        ],
                        "userIdentity": {"autoUser": {"elevationLevel": "str", "scope": "str"}, "userName": "str"},
                        "waitForSuccess": bool,
                    },
                    "taskSchedulingPolicy": {"nodeFillType": "str", "jobDefaultOrder": "str"},
                    "taskSlotsPerNode": 0,
                    "upgradePolicy": {
                        "mode": "str",
                        "automaticOSUpgradePolicy": {
                            "disableAutomaticRollback": bool,
                            "enableAutomaticOSUpgrade": bool,
                            "osRollingUpgradeDeferral": bool,
                            "useRollingUpgradePolicy": bool,
                        },
                        "rollingUpgradePolicy": {
                            "enableCrossZoneUpgrade": bool,
                            "maxBatchInstancePercent": 0,
                            "maxUnhealthyInstancePercent": 0,
                            "maxUnhealthyUpgradedInstancePercent": 0,
                            "pauseTimeBetweenBatches": "str",
                            "prioritizeUnhealthyInstances": bool,
                            "rollbackFailedInstancesOnPolicyBreach": bool,
                        },
                    },
                    "userAccounts": [
                        {
                            "name": "str",
                            "password": "str",
                            "elevationLevel": "str",
                            "linuxUserConfiguration": {"gid": 0, "sshPrivateKey": "str", "uid": 0},
                            "windowsUserConfiguration": {"loginMode": "str"},
                        }
                    ],
                    "vmSize": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_begin_delete(self, resource_group):
        response = await (
            await self.client.pool.begin_delete(
                resource_group_name=resource_group.name,
                account_name="str",
                pool_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_list_by_batch_account(self, resource_group):
        response = self.client.pool.list_by_batch_account(
            resource_group_name=resource_group.name,
            account_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_disable_auto_scale(self, resource_group):
        response = await self.client.pool.disable_auto_scale(
            resource_group_name=resource_group.name,
            account_name="str",
            pool_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_pool_stop_resize(self, resource_group):
        response = await self.client.pool.stop_resize(
            resource_group_name=resource_group.name,
            account_name="str",
            pool_name="str",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._cdn_management_client_operations import CdnManagementClientOperationsMixin
from ._afd_profiles_operations import AFDProfilesOperations
from ._afd_custom_domains_operations import AFDCustomDomainsOperations
from ._afd_endpoints_operations import AFDEndpointsOperations
from ._afd_origin_groups_operations import AFDOriginGroupsOperations
from ._afd_origins_operations import AFDOriginsOperations
from ._routes_operations import RoutesOperations
from ._rule_sets_operations import RuleSetsOperations
from ._rules_operations import RulesOperations
from ._security_policies_operations import SecurityPoliciesOperations
from ._secrets_operations import SecretsOperations
from ._log_analytics_operations import LogAnalyticsOperations
from ._profiles_operations import ProfilesOperations
from ._endpoints_operations import EndpointsOperations
from ._origins_operations import OriginsOperations
from ._origin_groups_operations import OriginGroupsOperations
from ._custom_domains_operations import CustomDomainsOperations
from ._resource_usage_operations import ResourceUsageOperations
from ._operations import Operations
from ._edge_nodes_operations import EdgeNodesOperations
from ._policies_operations import PoliciesOperations
from ._managed_rule_sets_operations import ManagedRuleSetsOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "CdnManagementClientOperationsMixin",
    "AFDProfilesOperations",
    "AFDCustomDomainsOperations",
    "AFDEndpointsOperations",
    "AFDOriginGroupsOperations",
    "AFDOriginsOperations",
    "RoutesOperations",
    "RuleSetsOperations",
    "RulesOperations",
    "SecurityPoliciesOperations",
    "SecretsOperations",
    "LogAnalyticsOperations",
    "ProfilesOperations",
    "EndpointsOperations",
    "OriginsOperations",
    "OriginGroupsOperations",
    "CustomDomainsOperations",
    "ResourceUsageOperations",
    "Operations",
    "EdgeNodesOperations",
    "PoliciesOperations",
    "ManagedRuleSetsOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import contextlib
import functools
import logging
import inspect
from collections import namedtuple
from threading import Lock

from .helpers import create_random_name, trim_kwargs_from_test_function

_logger = logging.getLogger(__name__)
# Core Utility


class AbstractPreparer(object):
    _cache_lock = Lock()
    _resource_cache = {}
    ResourceCacheEntry = namedtuple("ResourceCacheEntry", "resource_name kwargs preparer")

    def __init__(self, name_prefix, name_len, disable_recording=False):
        self.name_prefix = name_prefix
        self.name_len = name_len
        self.resource_moniker = None
        self.resource_random_name = None
        self.test_class_instance = None
        self.live_test = False
        self.disable_recording = disable_recording
        self._cache_key = (self.__class__.__name__,)
        self._use_cache = False
        self._aggregate_cache_key = None

    def _prepare_create_resource(self, test_class_instance, **kwargs):
        self.test_class_instance = test_class_instance

        # This latter conditional is to triage a specific failure mode:
        # If the first cached test run does not have any http traffic, a recording will not have been
        # generated, so in_recording will be True even if live_test is false, so a random name would be given.
        # In cached mode we need to avoid this because then for tests with recordings, they would not have a moniker.
        if test_class_instance.in_recording and not (
            not test_class_instance.is_live and test_class_instance.in_recording and self._use_cache
        ):
            resource_name = self.random_name
        else:
            resource_name = self.moniker

        _logger.debug("Creating resource %s for %s", resource_name, self.__class__.__name__)
        with self.override_disable_recording():
            retries = 4
            for i in range(retries):
                try:
                    parameter_update = self.create_resource(resource_name, **kwargs)
                    _logger.debug("Successfully created resource %s", resource_name)
                    break
                except Exception as e:
                    msg = "Preparer failure when creating resource {} for test {}: {}".format(
                        self.__class__.__name__, test_class_instance, e
                    )
                    while e:
                        try:
                            e = e.inner_exception
                        except AttributeError:
                            break
                    try:
                        msg += "\nDetailed error message: " + str(e.additional_properties["error"]["message"])
                    except (AttributeError, KeyError):
                        pass

                    _logger.error(msg)
                    raise Exception(msg)

        if parameter_update:
            kwargs.update(parameter_update)

        return resource_name, kwargs

    def __call__(self, fn):
        def _preparer_wrapper(test_class_instance, **kwargs):
            _logger.debug(
                "Entering preparer wrapper for %s and test %s", self.__class__.__name__, str(test_class_instance)
            )

            # If a child is cached we must use the same cached resource their equivalent parent did so all the deps line up
            child_is_cached = getattr(fn, "__use_cache", False)
            # Note: If it is ever desired to make caching inferred, remove this if/throw.
            # This ensures that a user must _very specifically say they want caching_ on an item and all parents.
            if not self._use_cache and child_is_cached:
                raise Exception(
                    """Preparer exception for test {}:\n Child preparers are cached, but parent {} is not.
You must specify use_cache=True in the preparer decorator""".format(
                        test_class_instance, self.__class__.__name__
                    )
                )
            self._use_cache |= child_is_cached
            _logger.debug("Child cache status for %s: %s", self.__class__.__name__, child_is_cached)

            # We must use a cache_key that includes our parents, so that we get a cached stack
            # matching the desired resource stack. (e.g. if parent resource has specific settings)
            try:
                aggregate_cache_key = (self._cache_key, kwargs["__aggregate_cache_key"])
            except KeyError:  # If we're at the root of the cache stack, start with our own key.
                aggregate_cache_key = self._cache_key
            kwargs["__aggregate_cache_key"] = aggregate_cache_key
            self._aggregate_cache_key = aggregate_cache_key
            _logger.debug("Aggregate cache key: %s", aggregate_cache_key)

            # If cache is enabled, and the cached resource exists, use it, otherwise create and store.
            if self._use_cache and aggregate_cache_key in AbstractPreparer._resource_cache:
                _logger.debug("Using cached resource for %s", self.__class__.__name__)
                with self._cache_lock:
                    new_kwargs = kwargs
                    resource_name, kwargs, _ = AbstractPreparer._resource_cache[aggregate_cache_key]
                    kwargs.update(new_kwargs)
            else:
                resource_name, kwargs = self._prepare_create_resource(test_class_instance, **kwargs)

            if self._use_cache:
                with self._cache_lock:
                    if aggregate_cache_key not in AbstractPreparer._resource_cache:
                        _logger.debug("Storing cached resource for %s", self.__class__.__name__)
                        AbstractPreparer._resource_cache[aggregate_cache_key] = AbstractPreparer.ResourceCacheEntry(
                            resource_name, kwargs, self
                        )

            if test_class_instance.is_live:
                # Adding this for new proxy testcase
                if hasattr(test_class_instance, "scrubber"):
                    test_class_instance.scrubber.register_name_pair(resource_name, self.moniker)
                else:
                    _logger.info(
                        "This test class instance has no scrubber, so the AbstractPreparer will not scrub any values "
                        "in recordings."
                    )

            # We shouldn't trim the same kwargs that we use for deletion,
            # we may remove some of the variables we needed to do the delete.
            trimmed_kwargs = {k: v for k, v in kwargs.items()}
            trim_kwargs_from_test_function(fn, trimmed_kwargs)

            try:
                try:
                    import asyncio
                except ImportError:
                    fn(test_class_instance, **trimmed_kwargs)
                else:
                    if inspect.iscoroutinefunction(fn):
                        asyncio.run(fn(test_class_instance, **trimmed_kwargs))
                    else:
                        fn(test_class_instance, **trimmed_kwargs)
            finally:
                # If we use cache we delay deletion for the end.
                # This won't guarantee deletion order, but it will guarantee everything delayed
                # does get deleted, in the worst case by getting rid of the RG at the top.
                if not (self._use_cache or child_is_cached):
                    # Russian Doll - the last declared resource to be deleted first.
                    self.remove_resource_with_record_override(resource_name, **kwargs)

        # _logger.debug("Setting up preparer stack for {}".format(self.__class__.__name__))
        setattr(_preparer_wrapper, "__is_preparer", True)
        # Inform the next step in the chain (our parent) that we're cached.
        if self._use_cache or getattr(fn, "__use_cache", False):
            setattr(_preparer_wrapper, "__use_cache", True)
        functools.update_wrapper(_preparer_wrapper, fn)
        return _preparer_wrapper

    @contextlib.contextmanager
    def override_disable_recording(self):
        if hasattr(self.test_class_instance, "disable_recording"):
            orig_enabled = self.test_class_instance.disable_recording
            self.test_class_instance.disable_recording = self.disable_recording
            yield
            self.test_class_instance.disable_recording = orig_enabled
        else:
            yield

    @property
    def moniker(self):
        if not self.resource_moniker:
            self.test_class_instance.test_resources_count += 1
            self.resource_moniker = "{}{:06}".format(self.name_prefix, self.test_class_instance.test_resources_count)
        return self.resource_moniker

    def create_random_name(self):
        return create_random_name(self.name_prefix, self.name_len)

    @property
    def random_name(self):
        if not self.resource_random_name:
            self.resource_random_name = self.create_random_name()
        return self.resource_random_name

    # The only other design idea I see that doesn't require each preparer to be instrumented
    # would be to have a decorator at the top that wraps the rest, but the user would have to define
    # the "cache key" themselves which seems riskier (As opposed to as below, where it's defined
    # locally that sku and location are the parameters that make a resource unique)
    # This also would prevent fine-grained caching where leaf resources are still created.
    def set_cache(self, enabled, *args):
        # can't use *args expansion directly into a tuple, py27 compat.
        self._cache_key = tuple([self.__class__.__name__] + list(args))
        self._use_cache = enabled

    def create_resource(self, name, **kwargs):  # pylint: disable=unused-argument,no-self-use
        return {}

    def remove_resource(self, name, **kwargs):  # pylint: disable=unused-argument
        pass

    def remove_resource_with_record_override(self, name, **kwargs):
        with self.override_disable_recording():
            self.remove_resource(name, **kwargs)

    @classmethod
    def _perform_pending_deletes(cls):
        _logger.debug("Perform all delayed resource removal.")
        for resource_name, kwargs, preparer in reversed([e for e in cls._resource_cache.values()]):
            try:
                _logger.debug("Performing delayed delete for: %s %s", preparer, resource_name)
                preparer.remove_resource_with_record_override(resource_name, **kwargs)
            except Exception as e:  # pylint: disable=broad-except
                # Intentionally broad exception to attempt to leave as few orphan resources as possible even on error.
                _logger.warning("Exception while performing delayed deletes (this can happen): %s", e)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.automation.aio import AutomationClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAutomationDscNodeConfigurationOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AutomationClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_delete(self, resource_group):
        response = await self.client.dsc_node_configuration.delete(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            node_configuration_name="str",
            api_version="2022-08-08",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_get(self, resource_group):
        response = await self.client.dsc_node_configuration.get(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            node_configuration_name="str",
            api_version="2022-08-08",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.dsc_node_configuration.begin_create_or_update(
                resource_group_name=resource_group.name,
                automation_account_name="str",
                node_configuration_name="str",
                parameters={
                    "configuration": {"name": "str"},
                    "incrementNodeConfigurationBuild": bool,
                    "name": "str",
                    "source": {
                        "hash": {"algorithm": "str", "value": "str"},
                        "type": "str",
                        "value": "str",
                        "version": "str",
                    },
                    "tags": {"str": "str"},
                },
                api_version="2022-08-08",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list_by_automation_account(self, resource_group):
        response = self.client.dsc_node_configuration.list_by_automation_account(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            api_version="2022-08-08",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import BasicPreparer
from testpreparer_async import BasicClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBasicAsync(BasicClientTestBaseAsync):
    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_create_or_update(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = await client.create_or_update(
            id=0,
            resource={"etag": "str", "id": 0, "name": "str", "orders": [{"detail": "str", "id": 0, "userId": 0}]},
        )

        # please add some check logic here by yourself
        # ...

    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_create_or_replace(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = await client.create_or_replace(
            id=0,
            resource={"etag": "str", "id": 0, "name": "str", "orders": [{"detail": "str", "id": 0, "userId": 0}]},
        )

        # please add some check logic here by yourself
        # ...

    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_get(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = await client.get(
            id=0,
        )

        # please add some check logic here by yourself
        # ...

    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_list(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = client.list()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_delete(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = await client.delete(
            id=0,
        )

        # please add some check logic here by yourself
        # ...

    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_export(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = await client.export(
            id=0,
            format="str",
        )

        # please add some check logic here by yourself
        # ...

    @BasicPreparer()
    @recorded_by_proxy_async
    async def test_export_all_users(self, basic_endpoint):
        client = self.create_async_client(endpoint=basic_endpoint)
        response = await client.export_all_users(
            format="str",
        )

        # please add some check logic here by yourself
        # ...



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Invocations protocol for Azure AI Hosted Agents.

This package provides an invocation protocol host as a subclass of
:class:`~azure.ai.agentserver.core.AgentServerHost`.

Quick start::

    from azure.ai.agentserver.invocations import InvocationAgentServerHost
    from starlette.responses import JSONResponse

    app = InvocationAgentServerHost()

    @app.invoke_handler
    async def handle(request):
        return JSONResponse({"ok": True})

    app.run()
"""
__path__ = __import__("pkgutil").extend_path(__path__, __name__)

from ._invocation import InvocationAgentServerHost
from ._version import VERSION

__all__ = ["InvocationAgentServerHost"]
__version__ = VERSION



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_utils.py
DESCRIPTION:
    Helper functions used for the azure attestation samples.

"""
import datetime
from cryptography.hazmat.backends import default_backend
from cryptography import x509
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509 import (
    BasicConstraints,
    CertificateBuilder,
    NameOID,
    SubjectAlternativeName,
)
from cryptography.hazmat.primitives import hashes, serialization


def write_banner(banner):
    # type: (str) -> None
    """
    Write a banner which can be used to visually separate the output of the samples.
    """
    separator = "*" * 80
    print("\n")
    print(separator)
    print("        {}".format(banner))
    print(separator)


def create_rsa_key() -> str:
    """
    Create an RSA Asymmetric 2048 bit key.
    """
    return (
        rsa.generate_private_key(65537, 2048, backend=default_backend())
        .private_bytes(
            serialization.Encoding.PEM,
            serialization.PrivateFormat.PKCS8,
            serialization.NoEncryption(),
        )
        .decode("ascii")
    )


def create_x509_certificate(key_pem, subject_name):  # type: (str, str) -> str
    """
    Given an RSA or ECDS private key, create a self-signed X.509 certificate
    with the specified subject name signed with that key.
    """
    signing_key = serialization.load_pem_private_key(key_pem.encode("ascii"), password=None, backend=default_backend())
    builder = CertificateBuilder()
    builder = builder.subject_name(
        x509.Name(
            [
                x509.NameAttribute(NameOID.COMMON_NAME, subject_name),
            ]
        )
    )
    builder = builder.issuer_name(
        x509.Name(
            [
                x509.NameAttribute(NameOID.COMMON_NAME, subject_name),
            ]
        )
    )

    one_day = datetime.timedelta(1, 0, 0)
    builder = builder.not_valid_before(datetime.datetime.today() - one_day)
    builder = builder.not_valid_after(datetime.datetime.today() + (one_day * 30))
    builder = builder.serial_number(x509.random_serial_number())
    builder = builder.public_key(signing_key.public_key())  # type: ignore
    builder = builder.add_extension(SubjectAlternativeName([x509.DNSName(subject_name)]), critical=False)
    builder = builder.add_extension(BasicConstraints(ca=False, path_length=None), critical=True)
    return (
        builder.sign(
            private_key=signing_key,  # type: ignore
            algorithm=hashes.SHA256(),
            backend=default_backend(),
        )
        .public_bytes(serialization.Encoding.PEM)
        .decode("ascii")
    )


def pem_from_base64(base64_value, header_type):
    # type: (str, str) -> str
    pem = "-----BEGIN " + header_type + "-----\n"
    while base64_value != "":
        pem += base64_value[:64] + "\n"
        base64_value = base64_value[64:]
    pem += "-----END " + header_type + "-----\n"
    return pem



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
Unit tests for verifying the base_url of the AsyncOpenAI client returned by AIProjectClient.get_openai_client().
No network calls are made.
"""

import pytest
from typing import Any
from azure.core.credentials_async import AsyncTokenCredential
from azure.ai.projects.aio import AIProjectClient

FAKE_ENDPOINT = "https://fake-account.services.ai.azure.com/api/projects/fake-project"
AGENT_NAME = "fake-agent-name"


class FakeAsyncCredential(AsyncTokenCredential):
    """Async stub credential that returns a never-expiring token."""

    async def get_token(self, *args: str, **kwargs: Any):  # type: ignore[override]
        from azure.core.credentials import AccessToken

        return AccessToken("fake-token", 9_999_999_999)

    async def close(self) -> None:
        pass


class TestGetOpenaiClientAsync:

    @pytest.mark.asyncio
    async def test_get_openai_client_default_endpoint_async(self):
        """Verify that the async OpenAI client base_url is set to {endpoint}/openai/v1."""
        project_client = AIProjectClient(
            endpoint=FAKE_ENDPOINT,
            credential=FakeAsyncCredential(),  # type: ignore[arg-type]
        )
        openai_client = project_client.get_openai_client()

        expected_base_url = FAKE_ENDPOINT.rstrip("/") + "/openai/v1"
        assert str(openai_client.base_url).rstrip("/") == expected_base_url

    @pytest.mark.asyncio
    async def test_get_openai_client_with_agent_name_raises_without_allow_preview_async(self):
        """Verify that passing agent_name without allow_preview=True raises ValueError."""
        project_client = AIProjectClient(
            endpoint=FAKE_ENDPOINT,
            credential=FakeAsyncCredential(),  # type: ignore[arg-type]
        )

        with pytest.raises(ValueError) as exc_info:
            project_client.get_openai_client(agent_name=AGENT_NAME)

        assert "allow_preview=True" in str(exc_info.value)

    @pytest.mark.asyncio
    async def test_get_openai_client_with_agent_name_and_allow_preview_async(self):
        """Verify that the async OpenAI client base_url includes the agent endpoint when allow_preview=True."""
        project_client = AIProjectClient(
            endpoint=FAKE_ENDPOINT,
            credential=FakeAsyncCredential(),  # type: ignore[arg-type]
            allow_preview=True,
        )
        openai_client = project_client.get_openai_client(agent_name=AGENT_NAME)

        expected_base_url = FAKE_ENDPOINT.rstrip("/") + f"/agents/{AGENT_NAME}/endpoint/protocols/openai"
        assert str(openai_client.base_url).rstrip("/") == expected_base_url
